var fs = require('fs');

var fileContent = fs.readFileSync(process.argv[2], {encoding: 'utf8'});

var dox = require('dox');
var util = require('util');

var _ = require('lodash');

function slug(str) {
  return str.replace(/\W+/g, '-').toLowerCase();
}

var comments = dox.parseComments(fileContent, {skipSingleStar: true})
  .filter(function(comment) {
    return !comment.isPrivate
  })
  .map(function(comment) {
    delete comment.code;

    var memberOf = _.where(comment.tags, {type: 'memberOf'})[0];
    var category = comment.category;
    var params = _.map(_.where(comment.tags, {type: 'param'}), 'name');

    var ctx = comment.ctx;

    comment.name = comment.tags.filter(function(tag) {
      return tag.type == 'name';
    })[0];
    comment.name = (comment.name && comment.name.value) || (ctx && ctx.name);

    comment.isStatic = !!comment.tags.filter(function(tag) {
      return tag.type == 'static';
    })[0];
    comment.callable = params.length > 0 || (ctx && ctx.type != 'property');

    comment.title = ((memberOf && ((memberOf.parent) + (comment.isStatic ? '.': '#'))) || '') + (comment.name || '' ) + ( comment.callable ? '(' + params.join(', ') + ')' : '');
    comment.shortTitle = ((memberOf && ((memberOf.parent) + (comment.isStatic ? '.': '#'))) || '') + (comment.name || '' );
    comment.slug = slug(comment.shortTitle);

    return comment;
  });
// console.log(util.inspect(comments, {depth: null}));


var tpl = fs.readFileSync('./doc.ejs', {encoding: 'utf8'});

function startsWith(str, prefix) {
  return str.substr(0, prefix.length)== prefix;
}

var c = _.sortBy(_.pairs(_.groupBy(comments, 'category')), function(c) {
  return c[0] == 'global' ? 1 : c[0] == 'should' ? 2 : startsWith(c[0], 'assertion') ? 3 : 10;
});
c = _.map(c, function(c) {
  return [c[0], _.sortBy(c[1])]
});
//console.log(util.inspect(c, {depth: null}));

var pkg = require('./package.json');

console.log(_.template(tpl, {version: pkg.version, commentsGrouped: c}));